"""Provides device automations for lights."""
import voluptuous as vol

from homeassistant.components.device_automation import toggle_entity
from homeassistant.const import CONF_DOMAIN
from . import DOMAIN


# mypy: allow-untyped-defs, no-check-untyped-defs

ACTION_SCHEMA = toggle_entity.ACTION_SCHEMA.extend({vol.Required(CONF_DOMAIN): DOMAIN})

CONDITION_SCHEMA = toggle_entity.CONDITION_SCHEMA.extend(
    {vol.Required(CONF_DOMAIN): DOMAIN}
)

TRIGGER_SCHEMA = toggle_entity.TRIGGER_SCHEMA.extend(
    {vol.Required(CONF_DOMAIN): DOMAIN}
)


async def async_call_action_from_config(hass, config, variables, context):
    """Change state based on configuration."""
    config = ACTION_SCHEMA(config)
    await toggle_entity.async_call_action_from_config(
        hass, config, variables, context, DOMAIN
    )


def async_condition_from_config(config, config_validation):
    """Evaluate state based on configuration."""
    config = CONDITION_SCHEMA(config)
    return toggle_entity.async_condition_from_config(config, config_validation)


async def async_trigger(hass, config, action, automation_info):
    """Listen for state changes based on configuration."""
    config = TRIGGER_SCHEMA(config)
    return await toggle_entity.async_attach_trigger(
        hass, config, action, automation_info
    )


async def async_get_actions(hass, device_id):
    """List device actions."""
    return await toggle_entity.async_get_actions(hass, device_id, DOMAIN)


async def async_get_conditions(hass, device_id):
    """List device conditions."""
    return await toggle_entity.async_get_conditions(hass, device_id, DOMAIN)


async def async_get_triggers(hass, device_id):
    """List device triggers."""
    return await toggle_entity.async_get_triggers(hass, device_id, DOMAIN)



"""Shared class to maintain Plex server instances."""
import logging

import plexapi.myplex
import plexapi.server
from requests import Session

from homeassistant.const import CONF_TOKEN, CONF_URL, CONF_VERIFY_SSL

from .const import CONF_SERVER, DEFAULT_VERIFY_SSL

_LOGGER = logging.getLogger(__package__)


class PlexServer:
    """Manages a single Plex server connection."""

    def __init__(self, server_config):
        """Initialize a Plex server instance."""
        self._plex_server = None
        self._url = server_config.get(CONF_URL)
        self._token = server_config.get(CONF_TOKEN)
        self._server_name = server_config.get(CONF_SERVER)
        self._verify_ssl = server_config.get(CONF_VERIFY_SSL, DEFAULT_VERIFY_SSL)

    def connect(self):
        """Connect to a Plex server directly, obtaining direct URL if necessary."""

        def _set_missing_url():
            account = plexapi.myplex.MyPlexAccount(token=self._token)
            available_servers = [
                x.name for x in account.resources() if "server" in x.provides
            ]
            server_choice = (
                self._server_name if self._server_name else available_servers[0]
            )
            connections = account.resource(server_choice).connections
            local_url = [x.httpuri for x in connections if x.local]
            remote_url = [x.uri for x in connections if not x.local]
            self._url = local_url[0] if local_url else remote_url[0]

        def _connect_with_url():
            session = None
            if self._url.startswith("https") and not self._verify_ssl:
                session = Session()
                session.verify = False
            self._plex_server = plexapi.server.PlexServer(
                self._url, self._token, session
            )
            _LOGGER.debug("Connected to: %s (%s)", self.friendly_name, self.url_in_use)

        if self._token and not self._url:
            _set_missing_url()

        _connect_with_url()

    def clients(self):
        """Pass through clients call to plexapi."""
        return self._plex_server.clients()

    def sessions(self):
        """Pass through sessions call to plexapi."""
        return self._plex_server.sessions()

    @property
    def friendly_name(self):
        """Return name of connected Plex server."""
        return self._plex_server.friendlyName

    @property
    def machine_identifier(self):
        """Return unique identifier of connected Plex server."""
        return self._plex_server.machineIdentifier

    @property
    def url_in_use(self):
        """Return URL used for connected Plex server."""
        return self._plex_server._baseurl  # pylint: disable=W0212



"""Device automation helpers for toggle entity."""
import voluptuous as vol

import homeassistant.components.automation.state as state
from homeassistant.components.device_automation.const import (
    CONF_IS_OFF,
    CONF_IS_ON,
    CONF_TOGGLE,
    CONF_TURN_OFF,
    CONF_TURN_ON,
    CONF_TURNED_OFF,
    CONF_TURNED_ON,
)
from homeassistant.core import split_entity_id
from homeassistant.const import (
    CONF_CONDITION,
    CONF_DEVICE_ID,
    CONF_DOMAIN,
    CONF_ENTITY_ID,
    CONF_PLATFORM,
    CONF_TYPE,
)
from homeassistant.helpers.entity_registry import async_entries_for_device
from homeassistant.helpers import condition, config_validation as cv, service

ENTITY_ACTIONS = [
    {
        # Turn entity off
        CONF_TYPE: CONF_TURN_OFF
    },
    {
        # Turn entity on
        CONF_TYPE: CONF_TURN_ON
    },
    {
        # Toggle entity
        CONF_TYPE: CONF_TOGGLE
    },
]

ENTITY_CONDITIONS = [
    {
        # True when entity is turned off
        CONF_CONDITION: "device",
        CONF_TYPE: CONF_IS_OFF,
    },
    {
        # True when entity is turned on
        CONF_CONDITION: "device",
        CONF_TYPE: CONF_IS_ON,
    },
]

ENTITY_TRIGGERS = [
    {
        # Trigger when entity is turned off
        CONF_PLATFORM: "device",
        CONF_TYPE: CONF_TURNED_OFF,
    },
    {
        # Trigger when entity is turned on
        CONF_PLATFORM: "device",
        CONF_TYPE: CONF_TURNED_ON,
    },
]

ACTION_SCHEMA = vol.Schema(
    {
        vol.Required(CONF_DEVICE_ID): str,
        vol.Required(CONF_DOMAIN): str,
        vol.Required(CONF_ENTITY_ID): cv.entity_id,
        vol.Required(CONF_TYPE): vol.In([CONF_TOGGLE, CONF_TURN_OFF, CONF_TURN_ON]),
    }
)

CONDITION_SCHEMA = vol.Schema(
    {
        vol.Required(CONF_CONDITION): "device",
        vol.Required(CONF_DEVICE_ID): str,
        vol.Required(CONF_DOMAIN): str,
        vol.Required(CONF_ENTITY_ID): cv.entity_id,
        vol.Required(CONF_TYPE): vol.In([CONF_IS_OFF, CONF_IS_ON]),
    }
)

TRIGGER_SCHEMA = vol.Schema(
    {
        vol.Required(CONF_PLATFORM): "device",
        vol.Required(CONF_DEVICE_ID): str,
        vol.Required(CONF_DOMAIN): str,
        vol.Required(CONF_ENTITY_ID): cv.entity_id,
        vol.Required(CONF_TYPE): vol.In([CONF_TURNED_OFF, CONF_TURNED_ON]),
    }
)


def _is_domain(entity, domain):
    return split_entity_id(entity.entity_id)[0] == domain


async def async_call_action_from_config(hass, config, variables, context, domain):
    """Change state based on configuration."""
    config = ACTION_SCHEMA(config)
    action_type = config[CONF_TYPE]
    if action_type == CONF_TURN_ON:
        action = "turn_on"
    elif action_type == CONF_TURN_OFF:
        action = "turn_off"
    else:
        action = "toggle"

    service_action = {
        service.CONF_SERVICE: "{}.{}".format(domain, action),
        CONF_ENTITY_ID: config[CONF_ENTITY_ID],
    }

    await service.async_call_from_config(
        hass, service_action, blocking=True, variables=variables, context=context
    )


def async_condition_from_config(config, config_validation):
    """Evaluate state based on configuration."""
    condition_type = config[CONF_TYPE]
    if condition_type == CONF_IS_ON:
        stat = "on"
    else:
        stat = "off"
    state_config = {
        condition.CONF_CONDITION: "state",
        condition.CONF_ENTITY_ID: config[CONF_ENTITY_ID],
        condition.CONF_STATE: stat,
    }

    return condition.state_from_config(state_config, config_validation)


async def async_attach_trigger(hass, config, action, automation_info):
    """Listen for state changes based on configuration."""
    trigger_type = config[CONF_TYPE]
    if trigger_type == CONF_TURNED_ON:
        from_state = "off"
        to_state = "on"
    else:
        from_state = "on"
        to_state = "off"
    state_config = {
        state.CONF_ENTITY_ID: config[CONF_ENTITY_ID],
        state.CONF_FROM: from_state,
        state.CONF_TO: to_state,
    }

    return await state.async_trigger(hass, state_config, action, automation_info)


async def _async_get_automations(hass, device_id, automation_templates, domain):
    """List device automations."""
    automations = []
    entity_registry = await hass.helpers.entity_registry.async_get_registry()

    entities = async_entries_for_device(entity_registry, device_id)
    domain_entities = [x for x in entities if _is_domain(x, domain)]
    for entity in domain_entities:
        for automation in automation_templates:
            automation = dict(automation)
            automation.update(
                device_id=device_id, entity_id=entity.entity_id, domain=domain
            )
            automations.append(automation)

    return automations


async def async_get_actions(hass, device_id, domain):
    """List device actions."""
    return await _async_get_automations(hass, device_id, ENTITY_ACTIONS, domain)


async def async_get_conditions(hass, device_id, domain):
    """List device conditions."""
    return await _async_get_automations(hass, device_id, ENTITY_CONDITIONS, domain)


async def async_get_triggers(hass, device_id, domain):
    """List device triggers."""
    return await _async_get_automations(hass, device_id, ENTITY_TRIGGERS, domain)



"""Models for permissions."""
from typing import TYPE_CHECKING

import attr

if TYPE_CHECKING:
    # pylint: disable=unused-import
    from homeassistant.helpers import entity_registry as ent_reg  # noqa
    from homeassistant.helpers import device_registry as dev_reg  # noqa


@attr.s(slots=True)
class PermissionLookup:
    """Class to hold data for permission lookups."""

    entity_registry = attr.ib(type="ent_reg.EntityRegistry")
    device_registry = attr.ib(type="dev_reg.DeviceRegistry")



"""Support for Obihai Sensors."""
import logging

from datetime import timedelta
import voluptuous as vol

from pyobihai import PyObihai

from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import (
    CONF_HOST,
    CONF_PASSWORD,
    CONF_USERNAME,
    DEVICE_CLASS_TIMESTAMP,
)

from homeassistant.helpers.entity import Entity
import homeassistant.helpers.config_validation as cv


_LOGGER = logging.getLogger(__name__)

SCAN_INTERVAL = timedelta(seconds=5)

OBIHAI = "Obihai"
DEFAULT_USERNAME = "admin"
DEFAULT_PASSWORD = "admin"

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
    {
        vol.Required(CONF_HOST): cv.string,
        vol.Optional(CONF_USERNAME, default=DEFAULT_USERNAME): cv.string,
        vol.Optional(CONF_PASSWORD, default=DEFAULT_PASSWORD): cv.string,
    }
)


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the Obihai sensor platform."""

    username = config[CONF_USERNAME]
    password = config[CONF_PASSWORD]
    host = config[CONF_HOST]

    sensors = []

    pyobihai = PyObihai()

    services = pyobihai.get_state(host, username, password)

    line_services = pyobihai.get_line_state(host, username, password)

    for key in services:
        sensors.append(ObihaiServiceSensors(pyobihai, host, username, password, key))

    for key in line_services:
        sensors.append(ObihaiServiceSensors(pyobihai, host, username, password, key))

    add_entities(sensors)


class ObihaiServiceSensors(Entity):
    """Get the status of each Obihai Lines."""

    def __init__(self, pyobihai, host, username, password, service_name):
        """Initialize monitor sensor."""
        self._host = host
        self._username = username
        self._password = password
        self._service_name = service_name
        self._state = None
        self._name = f"{OBIHAI} {self._service_name}"
        self._pyobihai = pyobihai

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._name

    @property
    def state(self):
        """Return the state of the sensor."""
        return self._state

    @property
    def device_class(self):
        """Return the device class for uptime sensor."""
        if self._service_name == "Last Reboot":
            return DEVICE_CLASS_TIMESTAMP
        return None

    def update(self):
        """Update the sensor."""
        services = self._pyobihai.get_state(self._host, self._username, self._password)

        if self._service_name in services:
            self._state = services.get(self._service_name)

        services = self._pyobihai.get_line_state(
            self._host, self._username, self._password
        )

        if self._service_name in services:
            self._state = services.get(self._service_name)



"""Support for Aqualink pool feature switches."""
import logging

from iaqualink import AqualinkToggle

from homeassistant.components.switch import DOMAIN, SwitchDevice
from homeassistant.config_entries import ConfigEntry
from homeassistant.helpers.typing import HomeAssistantType

from . import AqualinkEntity, refresh_system
from .const import DOMAIN as AQUALINK_DOMAIN

_LOGGER = logging.getLogger(__name__)

PARALLEL_UPDATES = 0


async def async_setup_entry(
    hass: HomeAssistantType, config_entry: ConfigEntry, async_add_entities
) -> None:
    """Set up discovered switches."""
    devs = []
    for dev in hass.data[AQUALINK_DOMAIN][DOMAIN]:
        devs.append(HassAqualinkSwitch(dev))
    async_add_entities(devs, True)


class HassAqualinkSwitch(SwitchDevice, AqualinkEntity):
    """Representation of a switch."""

    def __init__(self, dev: AqualinkToggle):
        """Initialize the switch."""
        self.dev = dev

    @property
    def name(self) -> str:
        """Return the name of the switch."""
        return self.dev.label

    @property
    def icon(self) -> str:
        """Return an icon based on the switch type."""
        if self.name == "Cleaner":
            return "mdi:robot-vacuum"
        if self.name == "Waterfall" or self.name.endswith("Dscnt"):
            return "mdi:fountain"
        if self.name.endswith("Pump") or self.name.endswith("Blower"):
            return "mdi:fan"
        if self.name.endswith("Heater"):
            return "mdi:radiator"

    @property
    def is_on(self) -> bool:
        """Return whether the switch is on or not."""
        return self.dev.is_on

    @refresh_system
    async def async_turn_on(self, **kwargs) -> None:
        """Turn on the switch."""
        await self.dev.turn_on()

    @refresh_system
    async def async_turn_off(self, **kwargs) -> None:
        """Turn off the switch."""
        await self.dev.turn_off()



"""Constants for the Plex component."""
DOMAIN = "plex"
NAME_FORMAT = "Plex {}"

DEFAULT_PORT = 32400
DEFAULT_SSL = False
DEFAULT_VERIFY_SSL = True

PLATFORMS = ["media_player", "sensor"]
SERVERS = "servers"

PLEX_CONFIG_FILE = "plex.conf"
PLEX_MEDIA_PLAYER_OPTIONS = "plex_mp_options"
PLEX_SERVER_CONFIG = "server_config"

CONF_SERVER = "server"
CONF_USE_EPISODE_ART = "use_episode_art"
CONF_SHOW_ALL_CONTROLS = "show_all_controls"



"""Helpers for device automations."""
import asyncio
import logging
from typing import Callable, cast

import voluptuous as vol

from homeassistant.components import websocket_api
from homeassistant.const import CONF_DOMAIN
from homeassistant.core import split_entity_id, HomeAssistant
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.entity_registry import async_entries_for_device
from homeassistant.helpers.typing import ConfigType
from homeassistant.loader import async_get_integration, IntegrationNotFound

DOMAIN = "device_automation"

_LOGGER = logging.getLogger(__name__)


async def async_setup(hass, config):
    """Set up device automation."""
    hass.components.websocket_api.async_register_command(
        websocket_device_automation_list_actions
    )
    hass.components.websocket_api.async_register_command(
        websocket_device_automation_list_conditions
    )
    hass.components.websocket_api.async_register_command(
        websocket_device_automation_list_triggers
    )
    return True


async def async_device_condition_from_config(
    hass: HomeAssistant, config: ConfigType, config_validation: bool = True
) -> Callable[..., bool]:
    """Wrap action method with state based condition."""
    if config_validation:
        config = cv.DEVICE_CONDITION_SCHEMA(config)
    integration = await async_get_integration(hass, config[CONF_DOMAIN])
    platform = integration.get_platform("device_automation")
    return cast(
        Callable[..., bool],
        platform.async_condition_from_config(config, config_validation),  # type: ignore
    )


async def _async_get_device_automations_from_domain(hass, domain, fname, device_id):
    """List device automations."""
    integration = None
    try:
        integration = await async_get_integration(hass, domain)
    except IntegrationNotFound:
        _LOGGER.warning("Integration %s not found", domain)
        return None

    try:
        platform = integration.get_platform("device_automation")
    except ImportError:
        # The domain does not have device automations
        return None

    if hasattr(platform, fname):
        return await getattr(platform, fname)(hass, device_id)


async def _async_get_device_automations(hass, fname, device_id):
    """List device automations."""
    device_registry, entity_registry = await asyncio.gather(
        hass.helpers.device_registry.async_get_registry(),
        hass.helpers.entity_registry.async_get_registry(),
    )

    domains = set()
    automations = []
    device = device_registry.async_get(device_id)
    for entry_id in device.config_entries:
        config_entry = hass.config_entries.async_get_entry(entry_id)
        domains.add(config_entry.domain)

    entities = async_entries_for_device(entity_registry, device_id)
    for entity in entities:
        domains.add(split_entity_id(entity.entity_id)[0])

    device_automations = await asyncio.gather(
        *(
            _async_get_device_automations_from_domain(hass, domain, fname, device_id)
            for domain in domains
        )
    )
    for device_automation in device_automations:
        if device_automation is not None:
            automations.extend(device_automation)

    return automations


@websocket_api.async_response
@websocket_api.websocket_command(
    {
        vol.Required("type"): "device_automation/action/list",
        vol.Required("device_id"): str,
    }
)
async def websocket_device_automation_list_actions(hass, connection, msg):
    """Handle request for device actions."""
    device_id = msg["device_id"]
    actions = await _async_get_device_automations(hass, "async_get_actions", device_id)
    connection.send_result(msg["id"], actions)


@websocket_api.async_response
@websocket_api.websocket_command(
    {
        vol.Required("type"): "device_automation/condition/list",
        vol.Required("device_id"): str,
    }
)
async def websocket_device_automation_list_conditions(hass, connection, msg):
    """Handle request for device conditions."""
    device_id = msg["device_id"]
    conditions = await _async_get_device_automations(
        hass, "async_get_conditions", device_id
    )
    connection.send_result(msg["id"], conditions)


@websocket_api.async_response
@websocket_api.websocket_command(
    {
        vol.Required("type"): "device_automation/trigger/list",
        vol.Required("device_id"): str,
    }
)
async def websocket_device_automation_list_triggers(hass, connection, msg):
    """Handle request for device triggers."""
    device_id = msg["device_id"]
    triggers = await _async_get_device_automations(
        hass, "async_get_triggers", device_id
    )
    connection.send_result(msg["id"], triggers)



"""Support for wake on lan."""
import logging
import platform
import subprocess as sp

import voluptuous as vol

from homeassistant.components.switch import PLATFORM_SCHEMA, SwitchDevice
from homeassistant.const import CONF_HOST, CONF_NAME
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.script import Script

_LOGGER = logging.getLogger(__name__)

CONF_BROADCAST_ADDRESS = "broadcast_address"
CONF_MAC_ADDRESS = "mac_address"
CONF_OFF_ACTION = "turn_off"

DEFAULT_NAME = "Wake on LAN"
DEFAULT_PING_TIMEOUT = 1

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
    {
        vol.Required(CONF_MAC_ADDRESS): cv.string,
        vol.Optional(CONF_BROADCAST_ADDRESS): cv.string,
        vol.Optional(CONF_HOST): cv.string,
        vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
        vol.Optional(CONF_OFF_ACTION): cv.SCRIPT_SCHEMA,
    }
)


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up a wake on lan switch."""
    broadcast_address = config.get(CONF_BROADCAST_ADDRESS)
    host = config.get(CONF_HOST)
    mac_address = config.get(CONF_MAC_ADDRESS)
    name = config.get(CONF_NAME)
    off_action = config.get(CONF_OFF_ACTION)

    add_entities(
        [WOLSwitch(hass, name, host, mac_address, off_action, broadcast_address)], True
    )


class WOLSwitch(SwitchDevice):
    """Representation of a wake on lan switch."""

    def __init__(self, hass, name, host, mac_address, off_action, broadcast_address):
        """Initialize the WOL switch."""
        import wakeonlan

        self._hass = hass
        self._name = name
        self._host = host
        self._mac_address = mac_address
        self._broadcast_address = broadcast_address
        self._off_script = Script(hass, off_action) if off_action else None
        self._state = False
        self._wol = wakeonlan

    @property
    def is_on(self):
        """Return true if switch is on."""
        return self._state

    @property
    def name(self):
        """Return the name of the switch."""
        return self._name

    def turn_on(self, **kwargs):
        """Turn the device on."""
        if self._broadcast_address:
            self._wol.send_magic_packet(
                self._mac_address, ip_address=self._broadcast_address
            )
        else:
            self._wol.send_magic_packet(self._mac_address)

    def turn_off(self, **kwargs):
        """Turn the device off if an off action is present."""
        if self._off_script is not None:
            self._off_script.run()

    def update(self):
        """Check if device is on and update the state."""
        if platform.system().lower() == "windows":
            ping_cmd = [
                "ping",
                "-n",
                "1",
                "-w",
                str(DEFAULT_PING_TIMEOUT * 1000),
                str(self._host),
            ]
        else:
            ping_cmd = [
                "ping",
                "-c",
                "1",
                "-W",
                str(DEFAULT_PING_TIMEOUT),
                str(self._host),
            ]

        status = sp.call(ping_cmd, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
        self._state = not bool(status)



"""Support to embed Plex."""
import logging

import plexapi.exceptions
import requests.exceptions
import voluptuous as vol

from homeassistant.components.discovery import SERVICE_PLEX
from homeassistant.components.media_player import DOMAIN as MP_DOMAIN
from homeassistant.const import (
    CONF_HOST,
    CONF_PORT,
    CONF_SSL,
    CONF_TOKEN,
    CONF_URL,
    CONF_VERIFY_SSL,
)
from homeassistant.helpers import config_validation as cv
from homeassistant.helpers import discovery
from homeassistant.util.json import load_json, save_json

from .const import (
    CONF_SERVER,
    CONF_USE_EPISODE_ART,
    CONF_SHOW_ALL_CONTROLS,
    DEFAULT_PORT,
    DEFAULT_SSL,
    DEFAULT_VERIFY_SSL,
    DOMAIN as PLEX_DOMAIN,
    PLATFORMS,
    PLEX_CONFIG_FILE,
    PLEX_MEDIA_PLAYER_OPTIONS,
    SERVERS,
)
from .server import PlexServer

MEDIA_PLAYER_SCHEMA = vol.Schema(
    {
        vol.Optional(CONF_USE_EPISODE_ART, default=False): cv.boolean,
        vol.Optional(CONF_SHOW_ALL_CONTROLS, default=False): cv.boolean,
    }
)

SERVER_CONFIG_SCHEMA = vol.Schema(
    vol.All(
        {
            vol.Optional(CONF_HOST): cv.string,
            vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.port,
            vol.Optional(CONF_TOKEN): cv.string,
            vol.Optional(CONF_SERVER): cv.string,
            vol.Optional(CONF_SSL, default=DEFAULT_SSL): cv.boolean,
            vol.Optional(CONF_VERIFY_SSL, default=DEFAULT_VERIFY_SSL): cv.boolean,
            vol.Optional(MP_DOMAIN, default={}): MEDIA_PLAYER_SCHEMA,
        },
        cv.has_at_least_one_key(CONF_HOST, CONF_TOKEN),
    )
)

CONFIG_SCHEMA = vol.Schema({PLEX_DOMAIN: SERVER_CONFIG_SCHEMA}, extra=vol.ALLOW_EXTRA)

CONFIGURING = "configuring"
_LOGGER = logging.getLogger(__package__)


def setup(hass, config):
    """Set up the Plex component."""

    def server_discovered(service, info):
        """Pass back discovered Plex server details."""
        if hass.data[PLEX_DOMAIN][SERVERS]:
            _LOGGER.debug("Plex server already configured, ignoring discovery.")
            return
        _LOGGER.debug("Discovered Plex server: %s:%s", info["host"], info["port"])
        setup_plex(discovery_info=info)

    def setup_plex(config=None, discovery_info=None, configurator_info=None):
        """Return assembled server_config dict."""
        json_file = hass.config.path(PLEX_CONFIG_FILE)
        file_config = load_json(json_file)
        host_and_port = None

        if config:
            server_config = config
            if CONF_HOST in server_config:
                host_and_port = (
                    f"{server_config.pop(CONF_HOST)}:{server_config.pop(CONF_PORT)}"
                )
            if MP_DOMAIN in server_config:
                hass.data[PLEX_MEDIA_PLAYER_OPTIONS] = server_config.pop(MP_DOMAIN)
        elif file_config:
            _LOGGER.debug("Loading config from %s", json_file)
            host_and_port, server_config = file_config.popitem()
            server_config[CONF_VERIFY_SSL] = server_config.pop("verify")
        elif discovery_info:
            server_config = {}
            host_and_port = f"{discovery_info[CONF_HOST]}:{discovery_info[CONF_PORT]}"
        elif configurator_info:
            server_config = configurator_info
            host_and_port = server_config["host_and_port"]
        else:
            discovery.listen(hass, SERVICE_PLEX, server_discovered)
            return True

        if host_and_port:
            use_ssl = server_config.get(CONF_SSL, DEFAULT_SSL)
            http_prefix = "https" if use_ssl else "http"
            server_config[CONF_URL] = f"{http_prefix}://{host_and_port}"

        plex_server = PlexServer(server_config)
        try:
            plex_server.connect()
        except requests.exceptions.ConnectionError as error:
            _LOGGER.error(
                "Plex server could not be reached, please verify host and port: [%s]",
                error,
            )
            return False
        except (
            plexapi.exceptions.BadRequest,
            plexapi.exceptions.Unauthorized,
            plexapi.exceptions.NotFound,
        ) as error:
            _LOGGER.error(
                "Connection to Plex server failed, please verify token and SSL settings: [%s]",
                error,
            )
            request_configuration(host_and_port)
            return False
        else:
            hass.data[PLEX_DOMAIN][SERVERS][
                plex_server.machine_identifier
            ] = plex_server

            if host_and_port in hass.data[PLEX_DOMAIN][CONFIGURING]:
                request_id = hass.data[PLEX_DOMAIN][CONFIGURING].pop(host_and_port)
                configurator = hass.components.configurator
                configurator.request_done(request_id)
                _LOGGER.debug("Discovery configuration done")
            if configurator_info:
                # Write plex.conf if created via discovery/configurator
                save_json(
                    hass.config.path(PLEX_CONFIG_FILE),
                    {
                        host_and_port: {
                            CONF_TOKEN: server_config[CONF_TOKEN],
                            CONF_SSL: use_ssl,
                            "verify": server_config[CONF_VERIFY_SSL],
                        }
                    },
                )

            if not hass.data.get(PLEX_MEDIA_PLAYER_OPTIONS):
                hass.data[PLEX_MEDIA_PLAYER_OPTIONS] = MEDIA_PLAYER_SCHEMA({})

            for platform in PLATFORMS:
                hass.helpers.discovery.load_platform(
                    platform, PLEX_DOMAIN, {}, original_config
                )

            return True

    def request_configuration(host_and_port):
        """Request configuration steps from the user."""
        configurator = hass.components.configurator
        if host_and_port in hass.data[PLEX_DOMAIN][CONFIGURING]:
            configurator.notify_errors(
                hass.data[PLEX_DOMAIN][CONFIGURING][host_and_port],
                "Failed to register, please try again.",
            )
            return

        def plex_configuration_callback(data):
            """Handle configuration changes."""
            config = {
                "host_and_port": host_and_port,
                CONF_TOKEN: data.get("token"),
                CONF_SSL: cv.boolean(data.get("ssl")),
                CONF_VERIFY_SSL: cv.boolean(data.get("verify_ssl")),
            }
            setup_plex(configurator_info=config)

        hass.data[PLEX_DOMAIN][CONFIGURING][
            host_and_port
        ] = configurator.request_config(
            "Plex Media Server",
            plex_configuration_callback,
            description="Enter the X-Plex-Token",
            entity_picture="/static/images/logo_plex_mediaserver.png",
            submit_caption="Confirm",
            fields=[
                {"id": "token", "name": "X-Plex-Token", "type": ""},
                {"id": "ssl", "name": "Use SSL", "type": ""},
                {"id": "verify_ssl", "name": "Verify SSL", "type": ""},
            ],
        )

    # End of inner functions.

    original_config = config

    hass.data.setdefault(PLEX_DOMAIN, {SERVERS: {}, CONFIGURING: {}})

    if hass.data[PLEX_DOMAIN][SERVERS]:
        _LOGGER.debug("Plex server already configured")
        return False

    plex_config = config.get(PLEX_DOMAIN, {})
    return setup_plex(config=plex_config)



"""Provides device automations for deconz events."""
import voluptuous as vol

import homeassistant.components.automation.event as event

from homeassistant.components.device_automation.exceptions import (
    InvalidDeviceAutomationConfig,
)
from homeassistant.const import (
    CONF_DEVICE_ID,
    CONF_DOMAIN,
    CONF_EVENT,
    CONF_PLATFORM,
    CONF_TYPE,
)

from . import DOMAIN
from .config_flow import configured_gateways
from .deconz_event import CONF_DECONZ_EVENT, CONF_UNIQUE_ID
from .gateway import get_gateway_from_config_entry

CONF_SUBTYPE = "subtype"

CONF_SHORT_PRESS = "remote_button_short_press"
CONF_SHORT_RELEASE = "remote_button_short_release"
CONF_LONG_PRESS = "remote_button_long_press"
CONF_LONG_RELEASE = "remote_button_long_release"
CONF_DOUBLE_PRESS = "remote_button_double_press"
CONF_TRIPLE_PRESS = "remote_button_triple_press"
CONF_QUADRUPLE_PRESS = "remote_button_quadruple_press"
CONF_QUINTUPLE_PRESS = "remote_button_quintuple_press"
CONF_ROTATED = "remote_button_rotated"
CONF_SHAKE = "remote_gyro_activated"

CONF_TURN_ON = "turn_on"
CONF_TURN_OFF = "turn_off"
CONF_DIM_UP = "dim_up"
CONF_DIM_DOWN = "dim_down"
CONF_LEFT = "left"
CONF_RIGHT = "right"
CONF_OPEN = "open"
CONF_CLOSE = "close"
CONF_BOTH_BUTTONS = "both_buttons"
CONF_BUTTON_1 = "button_1"
CONF_BUTTON_2 = "button_2"
CONF_BUTTON_3 = "button_3"
CONF_BUTTON_4 = "button_4"

HUE_DIMMER_REMOTE_MODEL = "RWL021"
HUE_DIMMER_REMOTE = {
    (CONF_SHORT_PRESS, CONF_TURN_ON): 1000,
    (CONF_SHORT_RELEASE, CONF_TURN_ON): 1002,
    (CONF_LONG_PRESS, CONF_TURN_ON): 1001,
    (CONF_LONG_RELEASE, CONF_TURN_ON): 1003,
    (CONF_SHORT_PRESS, CONF_DIM_UP): 2000,
    (CONF_SHORT_RELEASE, CONF_DIM_UP): 2002,
    (CONF_LONG_PRESS, CONF_DIM_UP): 2001,
    (CONF_LONG_RELEASE, CONF_DIM_UP): 2003,
    (CONF_SHORT_PRESS, CONF_DIM_DOWN): 3000,
    (CONF_SHORT_RELEASE, CONF_DIM_DOWN): 3002,
    (CONF_LONG_PRESS, CONF_DIM_DOWN): 3001,
    (CONF_LONG_RELEASE, CONF_DIM_DOWN): 3003,
    (CONF_SHORT_PRESS, CONF_TURN_OFF): 4000,
    (CONF_SHORT_RELEASE, CONF_TURN_OFF): 4002,
    (CONF_LONG_PRESS, CONF_TURN_OFF): 4001,
    (CONF_LONG_RELEASE, CONF_TURN_OFF): 4003,
}

HUE_TAP_REMOTE_MODEL = "ZGPSWITCH"
HUE_TAP_REMOTE = {
    (CONF_SHORT_PRESS, CONF_BUTTON_1): 34,
    (CONF_SHORT_PRESS, CONF_BUTTON_2): 16,
    (CONF_SHORT_PRESS, CONF_BUTTON_3): 17,
    (CONF_SHORT_PRESS, CONF_BUTTON_4): 18,
}

TRADFRI_ON_OFF_SWITCH_MODEL = "TRADFRI on/off switch"
TRADFRI_ON_OFF_SWITCH = {
    (CONF_SHORT_PRESS, CONF_TURN_ON): 1002,
    (CONF_LONG_PRESS, CONF_TURN_ON): 1001,
    (CONF_LONG_RELEASE, CONF_TURN_ON): 1003,
    (CONF_SHORT_PRESS, CONF_TURN_OFF): 2002,
    (CONF_LONG_PRESS, CONF_TURN_OFF): 2001,
    (CONF_LONG_RELEASE, CONF_TURN_OFF): 2003,
}

TRADFRI_OPEN_CLOSE_REMOTE_MODEL = "TRADFRI open/close remote"
TRADFRI_OPEN_CLOSE_REMOTE = {
    (CONF_SHORT_PRESS, CONF_OPEN): 1002,
    (CONF_LONG_PRESS, CONF_OPEN): 1003,
    (CONF_SHORT_PRESS, CONF_CLOSE): 2002,
    (CONF_LONG_PRESS, CONF_CLOSE): 2003,
}

TRADFRI_REMOTE_MODEL = "TRADFRI remote control"
TRADFRI_REMOTE = {
    (CONF_SHORT_PRESS, CONF_TURN_ON): 1002,
    (CONF_LONG_PRESS, CONF_TURN_ON): 1001,
    (CONF_SHORT_PRESS, CONF_DIM_UP): 2002,
    (CONF_LONG_PRESS, CONF_DIM_UP): 2001,
    (CONF_LONG_RELEASE, CONF_DIM_UP): 2003,
    (CONF_SHORT_PRESS, CONF_DIM_DOWN): 3002,
    (CONF_LONG_PRESS, CONF_DIM_DOWN): 3001,
    (CONF_LONG_RELEASE, CONF_DIM_DOWN): 3003,
    (CONF_SHORT_PRESS, CONF_LEFT): 4002,
    (CONF_LONG_PRESS, CONF_LEFT): 4001,
    (CONF_LONG_RELEASE, CONF_LEFT): 4003,
    (CONF_SHORT_PRESS, CONF_RIGHT): 5002,
    (CONF_LONG_PRESS, CONF_RIGHT): 5001,
    (CONF_LONG_RELEASE, CONF_RIGHT): 5003,
}

TRADFRI_WIRELESS_DIMMER_MODEL = "TRADFRI wireless dimmer"
TRADFRI_WIRELESS_DIMMER = {
    (CONF_ROTATED, CONF_LEFT): 3002,
    (CONF_ROTATED, CONF_RIGHT): 2002,
}

AQARA_DOUBLE_WALL_SWITCH_MODEL = "lumi.remote.b286acn01"
AQARA_DOUBLE_WALL_SWITCH = {
    (CONF_SHORT_PRESS, CONF_LEFT): 1002,
    (CONF_LONG_PRESS, CONF_LEFT): 1001,
    (CONF_DOUBLE_PRESS, CONF_LEFT): 1004,
    (CONF_SHORT_PRESS, CONF_RIGHT): 2002,
    (CONF_LONG_PRESS, CONF_RIGHT): 2001,
    (CONF_DOUBLE_PRESS, CONF_RIGHT): 2004,
    (CONF_SHORT_PRESS, CONF_BOTH_BUTTONS): 3002,
    (CONF_LONG_PRESS, CONF_BOTH_BUTTONS): 3001,
    (CONF_DOUBLE_PRESS, CONF_BOTH_BUTTONS): 3004,
}

AQARA_MINI_SWITCH_MODEL = "lumi.remote.b1acn01"
AQARA_MINI_SWITCH = {
    (CONF_SHORT_PRESS, CONF_TURN_ON): 1002,
    (CONF_DOUBLE_PRESS, CONF_TURN_ON): 1004,
    (CONF_LONG_PRESS, CONF_TURN_ON): 1001,
    (CONF_LONG_RELEASE, CONF_TURN_ON): 1003,
}

AQARA_ROUND_SWITCH_MODEL = "lumi.sensor_switch"
AQARA_ROUND_SWITCH = {
    (CONF_SHORT_PRESS, CONF_TURN_ON): 1000,
    (CONF_SHORT_RELEASE, CONF_TURN_ON): 1002,
    (CONF_DOUBLE_PRESS, CONF_TURN_ON): 1004,
    (CONF_TRIPLE_PRESS, CONF_TURN_ON): 1005,
    (CONF_QUADRUPLE_PRESS, CONF_TURN_ON): 1006,
    (CONF_QUINTUPLE_PRESS, CONF_TURN_ON): 1010,
    (CONF_LONG_PRESS, CONF_TURN_ON): 1001,
    (CONF_LONG_RELEASE, CONF_TURN_ON): 1003,
}

AQARA_SQUARE_SWITCH_MODEL = "lumi.sensor_switch.aq3"
AQARA_SQUARE_SWITCH = {
    (CONF_SHORT_PRESS, CONF_TURN_ON): 1002,
    (CONF_DOUBLE_PRESS, CONF_TURN_ON): 1004,
    (CONF_LONG_PRESS, CONF_TURN_ON): 1001,
    (CONF_LONG_RELEASE, CONF_TURN_ON): 1003,
    (CONF_SHAKE, ""): 1007,
}

REMOTES = {
    HUE_DIMMER_REMOTE_MODEL: HUE_DIMMER_REMOTE,
    HUE_TAP_REMOTE_MODEL: HUE_TAP_REMOTE,
    TRADFRI_ON_OFF_SWITCH_MODEL: TRADFRI_ON_OFF_SWITCH,
    TRADFRI_OPEN_CLOSE_REMOTE_MODEL: TRADFRI_OPEN_CLOSE_REMOTE,
    TRADFRI_REMOTE_MODEL: TRADFRI_REMOTE,
    TRADFRI_WIRELESS_DIMMER_MODEL: TRADFRI_WIRELESS_DIMMER,
    AQARA_DOUBLE_WALL_SWITCH_MODEL: AQARA_DOUBLE_WALL_SWITCH,
    AQARA_MINI_SWITCH_MODEL: AQARA_MINI_SWITCH,
    AQARA_ROUND_SWITCH_MODEL: AQARA_ROUND_SWITCH,
    AQARA_SQUARE_SWITCH_MODEL: AQARA_SQUARE_SWITCH,
}

TRIGGER_SCHEMA = vol.All(
    vol.Schema(
        {
            vol.Required(CONF_DEVICE_ID): str,
            vol.Required(CONF_DOMAIN): DOMAIN,
            vol.Required(CONF_PLATFORM): "device",
            vol.Required(CONF_TYPE): str,
            vol.Required(CONF_SUBTYPE): str,
        }
    )
)


def _get_deconz_event_from_device_id(hass, device_id):
    """Resolve deconz event from device id."""
    deconz_config_entries = configured_gateways(hass)
    for config_entry in deconz_config_entries.values():

        gateway = get_gateway_from_config_entry(hass, config_entry)
        for deconz_event in gateway.events:

            if device_id == deconz_event.device_id:
                return deconz_event

    return None


async def async_trigger(hass, config, action, automation_info):
    """Listen for state changes based on configuration."""
    config = TRIGGER_SCHEMA(config)

    device_registry = await hass.helpers.device_registry.async_get_registry()
    device = device_registry.async_get(config[CONF_DEVICE_ID])

    trigger = (config[CONF_TYPE], config[CONF_SUBTYPE])

    if device.model not in REMOTES and trigger not in REMOTES[device.model]:
        raise InvalidDeviceAutomationConfig

    trigger = REMOTES[device.model][trigger]

    deconz_event = _get_deconz_event_from_device_id(hass, device.id)
    if deconz_event is None:
        raise InvalidDeviceAutomationConfig

    event_id = deconz_event.serial

    state_config = {
        event.CONF_EVENT_TYPE: CONF_DECONZ_EVENT,
        event.CONF_EVENT_DATA: {CONF_UNIQUE_ID: event_id, CONF_EVENT: trigger},
    }

    return await event.async_trigger(hass, state_config, action, automation_info)


async def async_get_triggers(hass, device_id):
    """List device triggers.

    Make sure device is a supported remote model.
    Retrieve the deconz event object matching device entry.
    Generate device trigger list.
    """
    device_registry = await hass.helpers.device_registry.async_get_registry()
    device = device_registry.async_get(device_id)

    if device.model not in REMOTES:
        return

    triggers = []
    for trigger, subtype in REMOTES[device.model].keys():
        triggers.append(
            {
                CONF_DEVICE_ID: device_id,
                CONF_DOMAIN: DOMAIN,
                CONF_PLATFORM: "device",
                CONF_TYPE: trigger,
                CONF_SUBTYPE: subtype,
            }
        )

    return triggers



"""The pi_hole component."""
import logging

import voluptuous as vol
from hole import Hole
from hole.exceptions import HoleError

from homeassistant.const import CONF_HOST, CONF_NAME, CONF_SSL, CONF_VERIFY_SSL
from homeassistant.components.sensor import DOMAIN as SENSOR_DOMAIN
from homeassistant.helpers import config_validation as cv
from homeassistant.helpers.aiohttp_client import async_get_clientsession
from homeassistant.helpers.discovery import async_load_platform
from homeassistant.util import Throttle

from .const import (
    DOMAIN,
    CONF_LOCATION,
    DEFAULT_HOST,
    DEFAULT_LOCATION,
    DEFAULT_NAME,
    DEFAULT_SSL,
    DEFAULT_VERIFY_SSL,
    MIN_TIME_BETWEEN_UPDATES,
)

LOGGER = logging.getLogger(__name__)

CONFIG_SCHEMA = vol.Schema(
    {
        DOMAIN: vol.Schema(
            {
                vol.Optional(CONF_HOST, default=DEFAULT_HOST): cv.string,
                vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
                vol.Optional(CONF_SSL, default=DEFAULT_SSL): cv.boolean,
                vol.Optional(CONF_LOCATION, default=DEFAULT_LOCATION): cv.string,
                vol.Optional(CONF_VERIFY_SSL, default=DEFAULT_VERIFY_SSL): cv.boolean,
            }
        )
    },
    extra=vol.ALLOW_EXTRA,
)


async def async_setup(hass, config):
    """Set up the pi_hole integration."""

    conf = config[DOMAIN]
    name = conf[CONF_NAME]
    host = conf[CONF_HOST]
    use_tls = conf[CONF_SSL]
    verify_tls = conf[CONF_VERIFY_SSL]
    location = conf[CONF_LOCATION]

    LOGGER.debug("Setting up %s integration with host %s", DOMAIN, host)

    session = async_get_clientsession(hass, True)
    pi_hole = PiHoleData(
        Hole(
            host,
            hass.loop,
            session,
            location=location,
            tls=use_tls,
            verify_tls=verify_tls,
        ),
        name,
    )

    await pi_hole.async_update()

    hass.data[DOMAIN] = pi_hole

    hass.async_create_task(async_load_platform(hass, SENSOR_DOMAIN, DOMAIN, {}, config))

    return True


class PiHoleData:
    """Get the latest data and update the states."""

    def __init__(self, api, name):
        """Initialize the data object."""
        self.api = api
        self.name = name
        self.available = True

    @Throttle(MIN_TIME_BETWEEN_UPDATES)
    async def async_update(self):
        """Get the latest data from the Pi-hole."""

        try:
            await self.api.get_data()
            self.available = True
        except HoleError:
            LOGGER.error("Unable to fetch data from Pi-hole")
            self.available = False
